from unittest import TestCase
from laiye_dataservice import DataService, DSResult, Language
from tests import local_data
import time
import uuid


class BuiltInLoginTest(TestCase):

    def setUp(self) -> None:
        self.schema_name = BuiltInLoginTest.schema_name
        self.ds: DataService = BuiltInLoginTest.ds

    @staticmethod
    def setUpClass() -> None:
        BuiltInLoginTest.schema_name = 'dataservice_sdk_test'
        BuiltInLoginTest.ds = DataService(
            local_data.get_base_url(),
            local_data.get_auth_type(),
            username=local_data.get_username(),
            password=local_data.get_password(),
        )

    @staticmethod
    def tearDownClass() -> None:
        result: DSResult = BuiltInLoginTest.ds.get_schema(BuiltInLoginTest.schema_name)
        if result.get_code() != 200:
            return
        BuiltInLoginTest.ds.delete_schema(result.get_data().get("apiName"))
        BuiltInLoginTest.ds.logout()

    def test_00000_auth(self):
        ds = DataService(
            local_data.get_base_url(),
            'BuiltInUser',
            username=local_data.get_username(),
            password=local_data.get_password(),
        )

        result: DSResult = ds.get_schemas()
        self.assertEqual(result.get_code(), 200)
        self.assertTrue(len(result.get_data()) > 0)

        result: DSResult = ds.get_schema(uuid.uuid1())
        self.assertTrue(result.get_message().find('does not exist') > 0)

        ds.language = Language.zh_CN
        result: DSResult = ds.get_schema(uuid.uuid1())
        self.assertTrue(result.get_message().find('不存在') > 0)

    def test_00001_create_schema(self):
        result: DSResult = self.ds.create_schema(self.schema_name, self.schema_name, "")
        self.assertEqual(result.get_code(), 200)
        self.assertEqual(result.code, 200)
        self.assertEqual(result.get_data().get('apiName'), self.schema_name)
        self.assertIsNone(result.get_error_code())
        self.assertIsNotNone(result.get_message())
        self.assertIsNotNone(result.get_page_size())
        self.assertTrue(result.is_successfully())
        self.assertIsNotNone(result.get_json_data())

    def test_00010_add_fields(self):
        result: DSResult = self.ds.add_fields(
            self.schema_name,
            [
                {"allowNull": False, "apiName": "name", "dataMaxLength": 64, "dataMinLength": 2, "displayName": "name", "fieldType": "TEXT"},
                {"allowNull": False, "apiName": "address", "dataMaxLength": 64, "dataMinLength": 2, "displayName": "address", "fieldType": "TEXT"},
                {"allowNull": False, "apiName": "age", "displayName": "age", "fieldType": "LONG"}
            ]
        )

        self.assertEqual(result.get_code(), 200)
        self.assertEqual(len(result.get_data()), 3)

    def test_00020_add_fields(self):
        result: DSResult = self.ds.delete_fields(self.schema_name, ['address'])
        self.assertEqual(result.get_code(), 200)
        self.assertEqual(len(result.get_data()), 1)

    def test_00030_add_index(self):
        result: DSResult = self.ds.add_index(self.schema_name, {
          "apiName": "test_index",
          "displayName": "test_index",
          "fieldApiNames": ["name"],
          "type": "UNIQUE"
        })
        self.assertEqual(result.get_code(), 200)

        for _ in range(6):
            result: DSResult = self.ds.get_index(self.schema_name, 'test_index')
            self.assertEqual(result.get_code(), 200)
            if result.get_data().get('status') == 'SUCCESSFUL':
                break
            time.sleep(1)

    def test_00050_add_records(self):
        result: DSResult = self.ds.add_records(self.schema_name, [
            {"name": "Tom", "age": 100},
            {"name": "Jack", "age": 99},
        ], update_or_insert=True)

        self.assertEqual(result.get_code(), 200)

        result: DSResult = self.ds.add_record(self.schema_name, {"name": "Golf", "age": 90}, update_or_insert=True)

        self.assertEqual(result.get_code(), 200)

        result: DSResult = self.ds.get_records(self.schema_name)
        self.assertEqual(result.get_code(), 200)
        self.assertEqual(result.get_amount(), 3)

    def test_00060_exec_query_and_update(self):
        result: DSResult = self.ds.search_records(self.schema_name, {
            "filterConditions": [{"parameters": [0], "fieldApiName": "id", "operator": "GT"}],
            "sortConditions": [{"fieldApiName": "name", "sortType": "desc"}]
        })

        record_1 = result.get_data()[0]
        record_1["name"] = "Tom1"
        result1: DSResult = self.ds.update_record(self.schema_name, record_1.get('id'), record_1, response_with_data=True)

        record_2 = result.get_data()[1]
        record_2['name'] = 'Jack1'
        result2: DSResult = self.ds.update_records(self.schema_name, [record_2])

        result: DSResult = self.ds.exec_select_sql(f"select * from {self.schema_name} order by name desc")
        self.assertEqual(len(result.get_data().get("rows")), 3)

        self.assertEqual(result.get_data().get('rows')[0].get('name'), 'Tom1')
        self.assertEqual(result.get_data().get('rows')[1].get('name'), 'Jack1')

    def test_00070_delete_records(self):
        result: DSResult = self.ds.get_records(self.schema_name)

        ids = [record.get('id') for record in result.get_data()]

        result: DSResult = self.ds.delete_record(self.schema_name, ids[0])
        self.assertEqual(result.get_code(), 200)

        result: DSResult = self.ds.delete_records(self.schema_name, ids[1:])
        self.assertEqual(result.get_code(), 200)

        result: DSResult = self.ds.get_records(self.schema_name)
        self.assertEqual(result.get_amount(), 0)




from enum import Enum, unique


class AuthorizationType(Enum):
    UserCenter = 1
    BuiltInUser = 2
    JwtToken = 3

    @staticmethod
    def analysis(item):
        if isinstance(item, AuthorizationType):
            return item
        else:
            return AuthorizationType[item]


class HttpMethod(Enum):
    GET = 'get'
    POST = 'post'

    @staticmethod
    def analysis(item):
        if isinstance(item, HttpMethod):
            return item
        else:
            return HttpMethod[item]


@unique
class Language(Enum):
    EN = 'en'
    zh_CN = 'zh-CN'

    @staticmethod
    def analysis(item):
        if isinstance(item, Language):
            return item
        else:
            return Language[item]



from .enums import AuthorizationType


class UnsupportedAuthorizationType(Exception):
    def __init__(self, auth_type):
        super().__init__(
            f"Unsupported authorization type {auth_type}, it just support " 
            f"[{AuthorizationType.UserCenter}, {AuthorizationType.BuiltInUser}, {AuthorizationType.JwtToken}]"
        )



import logging
import requests
import json
from .enums import AuthorizationType, HttpMethod, Language
from .errors import UnsupportedAuthorizationType

logger = logging.getLogger(__name__)


class DSResult:
    def __init__(self, json_data):
        self.json_data = json_data

    def get_amount(self):
        return self.json_data.get("amount")

    def get_code(self):
        return self.json_data.get("code")

    def get_data(self):
        return self.json_data.get('data')

    def get_error_code(self):
        return self.json_data.get('errorCode')

    def get_message(self):
        return self.json_data.get('message')

    def get_page_size(self):
        return self.json_data.get('page_size')

    def is_successfully(self):
        return self.json_data.get('successfully', False)

    def get_json_data(self):
        return self.json_data

    def __getattr__(self, item):
        return self.json_data.get(item)


class DataService:
    def __init__(
            self,
            base_url,
            auth_type=AuthorizationType.UserCenter,
            username=None,
            password=None,
            api_key=None,
            timeout_seconds=30,
            language=Language.EN
    ):
        self.base_url = base_url
        self.auth_type = AuthorizationType.analysis(auth_type)

        self.api_key = api_key
        self.timeout_seconds = timeout_seconds
        self.language = Language.analysis(language)

        self.access_token = None
        self.authorization = None

        self.default_header = {"Content-Type": "application/json; charset=utf-8"}

        self.token = None

        if self.auth_type == AuthorizationType.UserCenter:
            pass
        elif self.auth_type == AuthorizationType.BuiltInUser:
            result = self.get_auth_token(username, password)
            if result.get_code() != 200 or result.get_data() is None:
                raise Exception("Get token error, please confirm your base_url, username or password")
                return
            self.token = result.get_data().get('token')
        elif self.auth_type == AuthorizationType.JwtToken:
            pass
        else:
            raise UnsupportedAuthorizationType(self.auth_type)

    def _get_authorized_header(self):
        header = self.default_header.copy()

        if self.auth_type == AuthorizationType.UserCenter:
            pass
        elif self.auth_type == AuthorizationType.BuiltInUser:
            header["Authorization"] = self.token
        elif self.auth_type == AuthorizationType.JwtToken:
            header["Jwt-Token"] = self.token
        else:
            raise UnsupportedAuthorizationType(self.auth_type)

        return header

    def _query(self, http_method, api_url, data=None, api_version='v1', if_need_auth=True):
        if if_need_auth:
            header = self._get_authorized_header()
        else:
            header = self.default_header.copy()

        header["Accept-Language"] = self.language.value

        url = f"{self.base_url}/api/{api_version}{api_url}"
        if http_method == HttpMethod.GET:
            response = requests.get(
                url,
                headers=header,
                timeout=self.timeout_seconds
            )
        elif http_method == HttpMethod.POST:
            response = requests.post(
                url,
                headers=header,
                data=json.dumps(data, ensure_ascii=False).encode('utf-8'),
                timeout=self.timeout_seconds
            )
        else:
            logger.error(f"unsupported request method {http_method}")
            raise Exception(f"unsupported request method {http_method}")

        return response

    @staticmethod
    def _resolve_response(
            res, if_log_error_when_not_200=True, if_resolve_json_body_when_not_200=False, url=''
    ) -> DSResult:
        if res is None:
            logger.error("Response is none")
            raise Exception("Response is none")

        if res.status_code != 200 and if_log_error_when_not_200:
            logger.error(f'Request to [{url}] failed, res status [{res.status_code}], res content [{res.text}]')

        if res.status_code == 200 or if_resolve_json_body_when_not_200:
            data = res.json()
            return DSResult(data)

        return DSResult({"code": res.status_code, "errorCode": 0, "successfully": False})

    """Auth"""
    def get_auth_token(self, username, password) -> DSResult:
        url = f'/user/auth'
        res = self._query(HttpMethod.POST, url, data={"username": username, "password": password}, if_need_auth=False)
        result = self._resolve_response(res, if_log_error_when_not_200=True, url=url)
        if result.get_code() != 200:
            logger.warning(
                f'get auth token error, message=[{result.get_message()}], status_code=[{result.get_code()}],'
                f' error_code={result.get_error_code()}'
            )
        return result

    def logout(self):
        url = f'/user/logout'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    """ Schema """
    def create_schema(self, api_name, display_name, description='') -> DSResult:
        url = f'/ddl/schema/add'
        data = {
            "apiName": api_name,
            "description": description,
            "displayName": display_name,
            "etag": 0,
            "ifEnableCls": False,
            "ifEnableRls": False
        }
        res = self._query(HttpMethod.POST, url, data=data)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def delete_schema(self, schema_api_name, expansion_level=2) -> DSResult:
        url = f'/ddl/schema/delete/{schema_api_name}?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def get_schemas(self, start_id=0, limit=1000, order='asc', expansion_level=2) -> DSResult:
        url = f'/ddl/schema/read/?startId={start_id}&limit={limit}&order={order}&expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def get_schema(self, schema_api_name, expansion_level=2) -> DSResult:
        url = f'/ddl/schema/read/{schema_api_name}?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def add_fields(self, schema_api_name, fields, expansion_level=2) -> DSResult:
        url = f'/ddl/schema/{schema_api_name}/field/add?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.POST, url, data=fields)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def delete_fields(self, schema_api_name, field_api_names, expansion_level=2) -> DSResult:
        url = f'/ddl/schema/{schema_api_name}/field/delete?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.POST, url, data=field_api_names)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def add_index(self, schema_api_name, index, expansion_level=2) -> DSResult:
        url = f'/ddl/schema/{schema_api_name}/index/add?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.POST, url, data=index)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def get_index(self, schema_api_name, index_api_name, expansion_level=2) -> DSResult:
        url = f'/ddl/schema/{schema_api_name}/index/read/{index_api_name}?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    """ Data """

    def add_record(self, schema_api_name, record, update_or_insert=False, response_with_data=False) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/add?upsert={update_or_insert}&responseWithData={response_with_data}'
        res = self._query(HttpMethod.POST, url, data=record)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def add_records(self, schema_api_name, records, update_or_insert=False, response_with_data=False) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/add-batch?upsert={update_or_insert}&responseWithData={response_with_data}'
        res = self._query(HttpMethod.POST, url, data=records)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def delete_records(self, schema_api_name, record_ids, response_with_data=False) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/delete-batch?responseWithData={response_with_data}'
        res = self._query(HttpMethod.POST, url, data=record_ids)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def delete_record(self, schema_api_name, record_id, response_with_data=False) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/delete/{record_id}?responseWithData={response_with_data}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def search_records(self, schema_api_name, search_data) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/query-data'
        res = self._query(HttpMethod.POST, url, data=search_data)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def get_records(self, schema_api_name, start_id=0, limit=1000, order='asc') -> DSResult:
        url = f'/dml/entity/{schema_api_name}/read?startId={start_id}&limit={limit}&order={order}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def update_records(self, schema_api_name, records, response_with_data=False) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/update-batch?responseWithData={response_with_data}'
        res = self._query(HttpMethod.POST, url, data=records)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def update_record(self, schema_api_name, record_id, record, response_with_data=False) -> DSResult:
        url = f'/dml/entity/{schema_api_name}/update/{record_id}?responseWithData={response_with_data}'
        res = self._query(HttpMethod.POST, url, data=record)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def exec_select_sql(self, select_sql) -> DSResult:
        url = f'/dml/sql/execute'
        res = self._query(HttpMethod.POST, url, data={"query": select_sql})
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)






import threading
from laiye_dataservice.enums import AuthorizationType

# 创建全局ThreadLocal对象:
_local_data = threading.local()

_base_url_key = 'base_url'
_auth_type_key = 'auth_type'
_username_key = 'username'
_password_key = 'password'


def clean_all_data():
    if is_exist_base_url():
        delattr(_local_data, _base_url_key)

    if is_exist_auth_type():
        delattr(_local_data, _auth_type_key)

    if is_exist_username():
        delattr(_local_data, _username_key)

    if is_exist_password():
        delattr(_local_data, _password_key)


# ---- Methods for user info ----

def is_exist_base_url() -> bool:
    return hasattr(_local_data, _base_url_key)


def set_base_url(user):
    setattr(_local_data, _base_url_key, user)


def get_base_url():
    if not is_exist_base_url():
        return None

    return getattr(_local_data, _base_url_key)


# ---- Methods for organization info ----

def is_exist_auth_type() -> bool:
    return hasattr(_local_data, _auth_type_key)


def set_auth_type(auth_type):
    setattr(_local_data, _auth_type_key, AuthorizationType[auth_type])


def get_auth_type():
    if not is_exist_auth_type():
        return None

    return getattr(_local_data, _auth_type_key)


# ---- Methods for organization info ----

def is_exist_username() -> bool:
    return hasattr(_local_data, _username_key)


def set_username(org):
    setattr(_local_data, _username_key, org)


def get_username():
    if not is_exist_username():
        return None

    return getattr(_local_data, _username_key)


# ---- Methods for organization info ----

def is_exist_password() -> bool:
    return hasattr(_local_data, _password_key)


def set_password(org):
    setattr(_local_data, _password_key, org)


def get_password():
    if not is_exist_password():
        return None

    return getattr(_local_data, _password_key)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="laiye-dataservice-sdk",
    version="1.6.3.01",
    author="kangyongsheng",
    author_email="kangyongsheng@laiye.com",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.laiye.com/laiye-dataservice/laiye-dataservice-sdk",
    packages=setuptools.find_packages(),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development",
    ],
    install_requires=[
        "requests", "PyJWT>=1.7.1"
    ],
    python_requires='>=3.6',
)
